from sqlalchemy.orm import Query


class SoftDeleteQuery(Query):
    def __new__(cls, *args, **kwargs):
        obj = super(SoftDeleteQuery, cls).__new__(cls)
        if len(args) > 0:
            super(SoftDeleteQuery, obj).__init__(*args, **kwargs)
            return obj.filter_by(deleted=False)
        return obj

    def filter(self, *criterion):
        return super(SoftDeleteQuery, self).filter(*criterion)


from setuptools import setup

setup(
  name='sqla-softdelete',
  packages=['sqla-softdelete'],
  version='1.0',
  license='GPLv3+',
  description='SQLAlchemy soft delete',
  author='Vitaly Efremov',
  author_email='efremov.vitaly@gmail.com',
  url='https://github.com/vitaly-efremov/sqla-softdelete',
  download_url='https://github.com/vitaly-efremov/sqla-softdelete/archive/1.0.tar.gz',
  keywords=['SQLAlchemy', 'Soft delete'],
  install_requires=[ 
          'sqlalchemy',
          'pytest',
  ],
  classifiers=[
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
  ],
)


from datetime import datetime

import sqlalchemy as sa
from sqlalchemy import event
from sqlalchemy import inspect
from sqlalchemy.orm import Query


class SoftDeleteMixin:
    deleted_at = sa.Column(sa.DateTime(timezone=True), nullable=True)

    def delete(self, deleted_at: datetime = None):
        self.deleted_at = deleted_at or datetime.now()


@event.listens_for(Query, 'before_compile', retval=True)
def before_compile(query):
    include_deleted = query._execution_options.get('include_deleted', False)
    if include_deleted:
        return query

    for column in query.column_descriptions:
        entity = column['entity']
        if entity is None:
            continue

        inspector = inspect(column['entity'])
        mapper = getattr(inspector, 'mapper', None)
        if mapper and issubclass(mapper.class_, SoftDeleteMixin):
            query = query.enable_assertions(False).filter(
                entity.deleted_at.is_(None),
            )

    return query


@event.listens_for(SoftDeleteMixin, 'load', propagate=True)
def load(obj, context):
    include_deleted = context.query._execution_options.get('include_deleted', False)
    if obj.deleted_at and not include_deleted:
        raise TypeError(f'Deleted object {obj} was loaded, did you use joined eager loading?')
